'use strict';

(function (global) {

    var ProfilePane = React.createClass({
        displayName: 'ProfilePane',

        componentDidMount: function componentDidMount() {
            this.props.pydio.UI.disableAllKeyBindings();
        },
        componentWillUnmount: function componentWillUnmount() {
            this.props.pydio.UI.enableAllKeyBindings();
        },

        getInitialState: function getInitialState() {
            var objValues = {};
            var pydio = this.props.pydio;
            pydio.user.preferences.forEach(function (v, k) {
                if (k === 'gui_preferences') return;
                objValues[k] = v;
            });
            return {
                definitions: PydioForm.Manager.parseParameters(pydio.getXmlRegistry(), "user/preferences/pref[@exposed='true']|//param[contains(@scope,'user') and @expose='true']"),
                values: objValues,
                originalValues: LangUtils.deepCopy(objValues),
                dirty: false
            };
        },

        onFormChange: function onFormChange(newValues, dirty, removeValues) {
            this.setState({ dirty: dirty, values: newValues });
        },

        getButton: function getButton(actionName, messageId) {
            var pydio = this.props.pydio;
            if (!pydio.Controller.getActionByName(actionName)) {
                return null;
            }
            var func = function func() {
                pydio.Controller.fireAction(actionName);
            };
            return React.createElement(ReactMUI.RaisedButton, { label: pydio.MessageHash[messageId], onClick: func });
        },

        revert: function revert() {
            this.setState({
                values: LangUtils.deepCopy(this.state.originalValues)
            });
        },

        saveForm: function saveForm() {
            if (!this.state.dirty) {
                this.setState({ dirty: false });
                return;
            }
            var pydio = this.props.pydio;
            var postValues = PydioForm.Manager.getValuesForPOST(this.state.definitions, this.state.values, 'PREFERENCES_');
            postValues['get_action'] = 'custom_data_edit';
            PydioApi.getClient().request(postValues, function (transport) {
                PydioApi.getClient().parseXmlMessage(transport.responseXML);
                global.document.observeOnce("ajaxplorer:registry_part_loaded", function (event) {
                    if (event.memo != "user/preferences") return;
                    pydio.Registry.logXmlUser(false);
                });
                pydio.loadXmlRegistry(false, "user/preferences");
                this.setState({ dirty: false });
            });
        },

        render: function render() {
            var pydio = this.props.pydio;

            var saveButton = React.createElement(ReactMUI.RaisedButton, { disabled: !this.state.dirty, label: pydio.MessageHash[53], onClick: this.saveForm });
            return React.createElement(
                'div',
                { className: 'react-mui-context' },
                React.createElement(
                    'div',
                    { className: 'title-flex' },
                    React.createElement(
                        'h3',
                        { style: { paddingLeft: 20 } },
                        pydio.MessageHash['user_dash.43'],
                        React.createElement(
                            'div',
                            { className: 'legend' },
                            pydio.MessageHash['user_dash.43t']
                        )
                    ),
                    React.createElement(
                        'div',
                        { className: 'actionBar' },
                        saveButton,
                        '  ',
                        this.getButton('pass_change', 194)
                    )
                ),
                React.createElement(PydioForm.FormPanel, {
                    parameters: this.state.definitions,
                    values: this.state.values,
                    depth: -1,
                    binary_context: "user_id=" + pydio.user.id,
                    onChange: this.onFormChange
                })
            );
        }

    });

    var WebDAVURL = React.createClass({
        displayName: 'WebDAVURL',

        propTypes: {
            url: React.PropTypes.string,
            label: React.PropTypes.string,
            getMessage: React.PropTypes.func
        },

        componentDidUpdate: function componentDidUpdate(prevProps, prevState) {
            this.attachClipboard();
        },

        componentDidMount: function componentDidMount() {
            this.attachClipboard();
        },

        componentWillUnmount: function componentWillUnmount() {
            this.detachClipboard();
        },

        detachClipboard: function detachClipboard() {
            if (this._clip) {
                this._clip.destroy();
            }
        },

        clearCopyMessage: function clearCopyMessage() {
            global.setTimeout((function () {
                this.setState({ copyMessage: '' });
            }).bind(this), 5000);
        },

        attachClipboard: function attachClipboard() {
            this.detachClipboard();
            if (this.refs['copy-button']) {
                this._clip = new Clipboard(this.refs['copy-button'].getDOMNode(), {
                    text: (function (trigger) {
                        return this.props.url;
                    }).bind(this)
                });
                this._clip.on('success', (function () {
                    this.setState({ copyMessage: this.props.getMessage('share_center.192') }, this.clearCopyMessage);
                }).bind(this));
                this._clip.on('error', (function () {
                    var copyMessage;
                    if (global.navigator.platform.indexOf("Mac") === 0) {
                        copyMessage = this.props.getMessage('share_center.144');
                    } else {
                        copyMessage = this.props.getMessage('share_center.143');
                    }
                    this.setState({ copyMessage: copyMessage }, this.clearCopyMessage);
                }).bind(this));
            }
        },

        render: function render() {
            var copy = undefined;
            if (this.state && this.state.copyMessage) {
                copy = React.createElement(
                    'div',
                    { className: 'copy_legend' },
                    this.state.copyMessage
                );
            }
            return React.createElement(
                'div',
                null,
                React.createElement(
                    'div',
                    { className: 'dav-url' },
                    React.createElement(ReactMUI.TextField, { floatingLabelText: this.props.label, value: this.props.url }),
                    React.createElement(ReactMUI.IconButton, { iconClassName: 'icon-paste', tooltip: this.props.getMessage('share_center.191'), ref: 'copy-button' })
                ),
                copy
            );
        }

    });

    var WebDAVPane = React.createClass({
        displayName: 'WebDAVPane',

        componentDidMount: function componentDidMount() {
            this.props.pydio.UI.disableAllKeyBindings();
            this.loadPrefs();
        },
        componentWillUnmount: function componentWillUnmount() {
            this.props.pydio.UI.enableAllKeyBindings();
        },
        getMessage: function getMessage(id) {
            return this.props.pydio.MessageHash[id];
        },
        onToggleChange: function onToggleChange(event, newValue) {
            PydioApi.getClient().request({
                get_action: 'webdav_preferences',
                activate: newValue ? "true" : "false"
            }, (function (t) {
                this.setState({ preferences: t.responseJSON });
                this.props.pydio.displayMessage("SUCCESS", this.props.pydio.MessageHash[newValue ? 408 : 409]);
            }).bind(this));
        },
        savePassword: function savePassword(event) {
            PydioApi.getClient().request({
                get_action: 'webdav_preferences',
                webdav_pass: this.refs['passfield'].getValue()
            }, (function (t) {
                this.setState({ preferences: t.responseJSON });
                this.props.pydio.displayMessage("SUCCESS", this.props.pydio.MessageHash[410]);
            }).bind(this));
        },
        loadPrefs: function loadPrefs() {
            if (!this.isMounted()) return;
            PydioApi.getClient().request({
                get_action: 'webdav_preferences'
            }, (function (t) {
                this.setState({ preferences: t.responseJSON });
            }).bind(this));
        },

        renderPasswordField: function renderPasswordField() {

            if (this.state.preferences.digest_set || !this.state.preferences.webdav_force_basic) {
                return null;
            }
            return React.createElement(
                'div',
                null,
                React.createElement(ReactMUI.TextField, {
                    type: 'password',
                    floatingLabelText: this.getMessage(523),
                    ref: 'passfield'
                }),
                '   ',
                React.createElement(ReactMUI.RaisedButton, {
                    label: 'Save',
                    onClick: this.savePassword
                }),
                React.createElement(
                    'div',
                    { className: 'dav-password-legend' },
                    this.getMessage(407)
                )
            );
        },

        renderURLList: function renderURLList() {

            var base = this.state.preferences.webdav_base_url;
            var userRepos = this.props.pydio.user.getRepositoriesList();
            var webdavRepos = this.state.preferences.webdav_repositories;
            var otherUrls = [];
            if (!!this.state.toggler) {
                userRepos.forEach((function (repo, key) {
                    if (!webdavRepos[key]) return;
                    otherUrls.push(React.createElement(WebDAVURL, { key: key, label: repo.getLabel(), url: webdavRepos[key], getMessage: this.getMessage }));
                }).bind(this));
            }

            var toggler = (function () {
                this.setState({ toggler: !this.state.toggler });
            }).bind(this);

            return React.createElement(
                'div',
                null,
                React.createElement(
                    'h4',
                    null,
                    'WebDAV shares adresses'
                ),
                React.createElement(
                    'div',
                    null,
                    this.getMessage(405)
                ),
                React.createElement(WebDAVURL, { label: this.getMessage(468), url: base, getMessage: this.getMessage }),
                React.createElement(
                    'div',
                    { style: { paddingTop: 20 } },
                    React.createElement(ReactMUI.Checkbox, { label: this.getMessage(465), onCheck: toggler, checked: !!this.state.toggler })
                ),
                otherUrls
            );
        },
        render: function render() {
            var webdavActive = this.state && this.state.preferences.webdav_active;
            return React.createElement(
                'div',
                { className: 'react-mui-context' },
                React.createElement(
                    'div',
                    { className: 'title-flex' },
                    React.createElement(
                        'h3',
                        { style: { paddingLeft: 20 } },
                        this.getMessage(403),
                        React.createElement(
                            'div',
                            { className: 'legend' },
                            this.getMessage(404)
                        )
                    )
                ),
                React.createElement(
                    'div',
                    { style: { padding: 20 } },
                    React.createElement(ReactMUI.Toggle, {
                        label: this.getMessage(406),
                        toggled: webdavActive,
                        onToggle: this.onToggleChange }),
                    React.createElement(
                        'div',
                        { style: { paddingTop: 20 } },
                        webdavActive ? this.renderPasswordField() : null,
                        webdavActive ? this.renderURLList() : null
                    )
                )
            );
        }

    });

    var ns = global.UserAccount || {};
    ns.ProfilePane = ProfilePane;
    ns.WebDAVPane = WebDAVPane;
    global.UserAccount = ns;
})(window);
